import io
import os
import speech_recognition as sr
import I2C_LCD_driver
import csv
import string
import RPi.GPIO as GPIO
import random
import threading
import pydub
import simpleaudio as sa
    
from datetime import datetime, timedelta
from queue import Queue
from tempfile import NamedTemporaryFile
from time import *
from pydub import AudioSegment
from pydub.playback import _play_with_simpleaudio


def main():
    
    #sleep(5) #for network to connect on boot
    
    badlist = []
    
    soundlist = ["oink1.wav","oink2.wav","oink3.wav", "oink4.wav", "oink5.wav", "oink6.wav", "oink7.wav", "oink8.wav", "oink9.wav", "oink10.wav", "goatScream.wav"]

    with open("bad.csv", newline='') as csvfile:
        spamreader = csv.reader(csvfile, delimiter=' ')
        for row in spamreader:
            badlist.append(row[0])

    
    record_timeout = 4 
    phrase_timeout = 3 # how much empty spac between recordings for new line
    energy_threshold = 800 # energy level for mic to detect
    
    numSwears = 0
    numCoins = 0
    
    mylcd = I2C_LCD_driver.lcd()
    
    print("display intilized")
    mylcd.lcd_clear()
    
    BEAM_PIN = 23
    
    print("data loaded")
    
    GPIO.setmode(GPIO.BCM)
    GPIO.setup(BEAM_PIN, GPIO.IN, pull_up_down=GPIO.PUD_UP)
    GPIO.add_event_detect(BEAM_PIN, GPIO.FALLING)
    
    sound = AudioSegment.from_wav(soundlist[random.randint(0,2)])

    temp_file = NamedTemporaryFile().name
    transcription = ['']
    transcription.append(' ')

    # the last time a recording was retreived from the queue.
    phrase_time = None
    # current raw audio bytes.
    last_sample = bytes()
    # thread safe Queue for passing data from threaded recording callback.
    data_queue = Queue()

    recorder = sr.Recognizer()
    recorder.energy_threshold = energy_threshold
    recorder.dynamic_energy_threshold = False
    
    def record_callback(_, audio:sr.AudioData) -> None:
        # threaded callback function to recieve audio data when recordings finish.
        # get raw bytes and push it into the thread safe queue.
        data = audio.get_raw_data()
        data_queue.put(data)

    #source = sr.Microphone(sample_rate=16000)
    with sr.Microphone(sample_rate=16000) as source:
        recorder.adjust_for_ambient_noise(source)

    


    #background thread that passes raw audio bytes.
    recorder.listen_in_background(source, record_callback, phrase_time_limit=record_timeout)
    print("model loaded.\n")
    
    mylcd.lcd_display_string("model loaded", 1)
    sleep(3)
    mylcd.lcd_display_string("Swears Sworn: " + str(numSwears), 1)
    mylcd.lcd_display_string("Coins: " + str(numCoins), 2)
    
    while True:
        try:
            now = datetime.utcnow()
            # Pull raw recorded audio from the queue
            if not data_queue.empty():
                phrase_complete = False
                # if enough time has passed between recordings, consider the phrase complete.
                # clear current working audio buffer to start over with the new data.
                if phrase_time and now - phrase_time > timedelta(seconds=phrase_timeout):
                    last_sample = bytes()
                    phrase_complete = True
                # the last time we received new audio data from the queue.
                phrase_time = now

                # concatenate current audio data with the latest audio data.
                while not data_queue.empty():
                    data = data_queue.get()
                    last_sample += data

                # Use AudioData to convert raw data to wav data.
                audio_data = sr.AudioData(last_sample, source.SAMPLE_RATE, source.SAMPLE_WIDTH)
                wav_data = io.BytesIO(audio_data.get_wav_data())

                # write wav data to the temp file as bytes.
                with open(temp_file, 'w+b') as f:
                    f.write(wav_data.read())

                # read the transcription.

                try:
                    result = recorder.recognize_google(audio_data)
                    text = result.strip()
                except sr.UnknownValueError:
                    print("???\n")
                    text = " "
                except sr.RequestError:
                    print("request error")
                    mylcd.lcd_clear()
                    mylcd.lcd_display_string("Request Error", 1)
                    mylcd.lcd_display_string("Check Internet", 2)
                    sleep(3)
                    mylcd.lcd_clear()
                    print("retrying")
                    mylcd.lcd_display_string("Retrying", 1)
                    sleep(3)
                    continue
                    
                # if detected a pause between recordings, add a new item to transcripion.
                # otherwise edit the existing one.
                if phrase_complete:
                    transcription.append(text)
                else:
                    transcription[-1] = text
                
                # clear console to reprint the updated transcription.
                os.system('cls' if os.name=='nt' else 'clear')
                print(text)
                for line in transcription:
                    for bad in badlist: 
                        sound = AudioSegment.from_wav(soundlist[random.randint(0,10)])
                        if(line.__contains__(bad)):
                            print('swear')
                            numSwears = numSwears + 1
                            mylcd.lcd_display_string("Swears Sworn: " + str(numSwears), 1)
                            transcription.clear()
                            transcription.append(' ')
                            #playback = _play_with_simpleaudio(sound * 50)
                           
                            
                            while True:
                                if GPIO.event_detected(BEAM_PIN):
                                    playback.stop()
                                    numCoins += 1
                                    mylcd.lcd_display_string("Coins: " + str(numCoins), 2)
                                    print("coin")
                                    break
                # flush stdout.
                print('', end='', flush=True)

                # prevent infinite loop
                sleep(0.25)
        except KeyboardInterrupt:
            GPIO.cleanup()
            break



if __name__ == "__main__":
    main()
